package de.clab.osami.conferencing.vlc.conferencingSystem;

import java.awt.Dimension;
import java.awt.Panel;


import uk.co.caprica.vlcj.log.Logger;
import uk.co.caprica.vlcj.version.LibVlcVersion;
import uk.co.caprica.vlcj.version.VlcjVersion;

import de.clab.osami.conferencing.vlc.ServiceInterfaces.IVideoConference;
import de.clab.osami.conferencing.vlc.ServiceInterfaces.VcConfig;
import de.clab.osami.conferencing.vlc.ServiceInterfaces.VcControl;
import de.clab.osami.conferencing.vlc.config.Configuration;
import de.clab.osami.conferencing.vlc.config.IConfiguration;
import de.clab.osami.conferencing.vlc.config.IStorageManager;
import de.clab.osami.conferencing.vlc.config.StorageManager;
import de.clab.osami.conferencing.vlc.ui.ConfigurationPanel;
import de.clab.osami.conferencing.vlc.util.Util;
import de.clab.osami.conferencing.vlc.util.VcLogger;

public class ConferencingSystem implements VcControl, VcConfig, IVideoConference {

	private IConfiguration config;
	private IProcessor processor;
	private IStorageManager storageManager;
	private ConfigurationPanel configPanel;
//	private final static Logger log = Logger.getLogger(ConferencingSystem.class);
	
	public ConferencingSystem(IProcessor processor)
	{
		VcLogger.init();
		Logger.setLevel(Logger.Level.INFO);
		Logger.info("VLC Version is "+LibVlcVersion.getVersion());
		this.processor = processor;
		storageManager = StorageManager.getInstance();
		config = storageManager.loadConfiguration();
				
		if(config!=null)
		{
			processor.init(config);
			//processor.configure();
		}
		else{

			config = Configuration.getInstance();
			processor.init(config);
			autoConfig();
			//processor.configure();
		}
		configPanel = new ConfigurationPanel(config);
	}
	
	public String reconfigure()
	{
		processor.configure();
		return null;
	}
	
	public String loadConfiguration(String location)
	{
		if(location == null)
			config = storageManager.loadConfiguration();
		else config= storageManager.loadConfiguration(location);
		processor.configure();
		return null;
	}
	
	public String saveConfiguration(String location)
	{
		if(location == null)
			storageManager.saveConfiguration(config);
		else storageManager.saveConfiguration(config,location);
		return null;
	}
	
	public String start(){
		return processor.start();
		//TODO handle case when camera is not connected
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.linux.vlc.conferencingSystem.IVideoConference#start(java.lang.String)
	 */
	public String start(String ip)
	{
		String result="";
		result = setIp(ip);
		return result+start();
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.linux.vlc.conferencingSystem.IVideoConference#stop()
	 */
	public String stop(){
		return processor.stop();
	}
	
	public IConfiguration getCofiguration()
	{
		return config;
	}
	
	
	protected String autoConfig() {
		return processor.autoConfig();
	}

	@Override
	public int getAudioBitrate() {
		return config.getAudioBitrate();
	}

	@Override
	public int getAudioChannels() {
		return config.getAudioChannelSize();
	}

	@Override
	public String[] getAudioInputDevices() {
		return Util.listToArray(processor.getAudioDevices());
	}

	@Override
	public int getAudioSampleRate() {
		return config.getAudioSampleRate();
	}

	@Override
	public int getCaptureCaching() {
		return config.getAudioCaching();
	}

	@Override
	public Panel getConfigurationPanel() {
		return configPanel;
	}

	@Override
	public String getCurrentAudioCodec() {
		return config.getAudioCodec();
	}

	@Override
	public String getCurrentAudioInputDevice() {
		return config.getAudioDevice();
	}

	@Override
	public String getCurrentVideoCodec() {
		return config.getVideoCodec();
	}

	@Override
	public String getCurrentVideoInputDevice() {
		return config.getVideoDevice();
	}

	@Override
	public String getIp() {
		return config.getStreamIp();
	}

	@Override
	public int getLocalPort() {
		return config.getStreamLocalPort();
	}

	@Override
	public int getReceiveCaching() {
		return config.getStreamCaching();
	}

	@Override
	public String[] getSupportedAudioCodecs() {

		return Util.listToArray(processor.getAudioCodecs());
	}

	@Override
	public String[] getSupportedVideoCodecs() {
		
		return Util.listToArray(processor.getVideoCodecs());
	}

	@Override
	public int getTargetPort() {
		
		return config.getStreamLocalPort();
	}

	@Override
	public int getVideoBitrate() {
		
		return config.getVideoBitrate();
	}

	@Override
	public int getVideoFrameRate() {
		
		return config.getVideoFrameRate();
	}

	@Override
	public String[] getVideoInputDevices() {
		
		return Util.listToArray(processor.getVideoDevices());
	}

	@Override
	public Object getVideoLocalComponent() {
		return processor.getLocalVisualComponent();
	}

	@Override
	public Object getVideoRemoteComponent() {
		return processor.getRemoteVisualComponent();

	}

	@Override
	public float getVideoScale() {
		
		return config.getVideoScale();
	}

	@Override
	public Dimension getVideoSize() {
		
		return config.getVideoSize();
	}

	@Override
	public boolean isAudioEnabled() {
		
		return config.isAudioEnabled();
	}


	@Override
	public String setAudioBitRate(int bitrate) {
		
		config.setAudioBitrate(bitrate);
		return null;
	}

	@Override
	public String setAudioChannels(byte channels) {
		
		config.setAudioChannelSize(channels);
		return null;
	}

	@Override
	public String setAudioCodec(String codec) {
		
		config.setAudioCodec(codec);
		return null;
	}

	@Override
	public String setAudioEnabled(boolean flag) {
		
		config.setAudioEnabled(flag);
		return null;
	}

	@Override
	public String setAudioInputDevice(String device) {
		
		config.setAudioDevice(device);
		return null;
	}

	@Override
	public String setAudioSampleRate(int sampleRate) {
		
		config.setAudioSampleRate(sampleRate);
		return null;
	}

	@Override
	public String setCaptureCaching(int ms) {
		
		config.setVideoCaching(ms);
		config.setAudioCaching(ms); 
		return null;
	}

	@Override
	public String setIp(String ipv4) {
		
		config.setStreamIp(ipv4);
		return null;
	}

	@Override
	public String setLocalPort(int port) {
		
		config.setStreamLocalPort(port);
		return null;
	}

	@Override
	public String setLoggingEnabled(boolean flag) {
		
		VcLogger.setEnabled(flag);
		return null;
	}

	@Override
	public String setReceiveCaching(int ms) {
		
		config.setStreamCaching(ms);
		return null;
	}

	@Override
	public String setTargetPort(int port) {
		
		config.setStreamTargetPort(port);
		return null;
	}

	@Override
	public String setVideoBitRate(int bitrate) {
		
		config.setVideoBitrate(bitrate);
		return null;
	}

	@Override
	public String setVideoCodec(String codec) {
		
		config.setVideoCodec(codec);
		return null;
	}

	@Override
	public String setVideoEnabled(boolean flag) {
		
		config.setVideoEnabled(flag);
		return null;
	}

	@Override
	public String setVideoFrameRate(int framerate) {
		
		config.setVideoFrameRate(framerate);
		return null;
	}

	@Override
	public String setVideoInputDevice(String device) {
		
		config.setVideoDevice(device);
		return null;
	}

	@Override
	public String setVideoLocalComponent(Object p) {
		//TODO set video surface
		return processor.setLocalVisualComponent(p);
	}


	@Override
	public String setVideoRemoteComponent(Object p) {
		//TODO set video surface
		return processor.setRemoteVisualComponent(p);
	}

	@Override
	public String setVideoScale(float scale) {
		
		config.setVideoScale(scale);
		return null;
	}

	@Override
	public String setVideoSize(Dimension size) {
		
		config.setVideoSize(size);
		return null;
	}

	@Override
	public boolean isVideoEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * @return
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getLocalIp()
	 */
	public String getStreamingLocalIp() {
		return config.getLocalIp();
	}

	/**
	 * @param localIp
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setLocalIp(java.lang.String)
	 */
	public void setStreamingLocalIp(String localIp) {
		config.setLocalIp(localIp);
	}

	@Override
	public void setVideoLocalEnabled(Boolean flag) {
		config.setVideoLocalEnabled(flag);
		
	}

	@Override
	public void setVideoRemoteEnabled(Boolean flag) {
		config.setVideoRemoteEnabled(flag);
		
	}
	
}
