package de.clab.osami.conferencing.vlc.config;

import java.awt.Dimension;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.config.items.AudioCaptureInfo;
import de.clab.osami.conferencing.vlc.config.items.AudioTranscoding;
import de.clab.osami.conferencing.vlc.config.items.StreamingInfo;
import de.clab.osami.conferencing.vlc.config.items.VideoCaptureInfo;
import de.clab.osami.conferencing.vlc.config.items.VideoTranscoding;

/**
 * @author christian
 *
 */

public class Configuration implements IConfiguration,IObservable{
	
	private AudioCaptureInfo audioCaptureInfo;
	private VideoCaptureInfo videoCaptureInfo;
	private VideoTranscoding videoTranscoding;
	private AudioTranscoding audioTranscoding;
	private StreamingInfo streamingInfo;
	private ArrayList<IConfigurationChangeListener> listener;
	private final static Logger log = Logger.getLogger(Configuration.class);
	private static IConfiguration config;
	private boolean videoRemoteEnabled;
	private boolean videoLocalEnabled;
	
	private Configuration()
	{
		audioCaptureInfo = new AudioCaptureInfo();
		videoCaptureInfo = new VideoCaptureInfo();
		videoTranscoding = new VideoTranscoding();
		audioTranscoding = new AudioTranscoding();
		streamingInfo = new StreamingInfo();
		listener = new ArrayList<IConfigurationChangeListener>();
		log.debug("new Configuraton Object created");
	}
	
	public static IConfiguration getInstance()
	{
		if(config == null)
			config = new Configuration();
		return config;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioCaching()
	 */
	public int getAudioCaching() {
		return audioCaptureInfo.getCaching();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioDevice()
	 */
	public String getAudioDevice() {
		return audioCaptureInfo.getDevice();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioProtocol()
	 */
	public String getAudioProtocol() {
		return audioCaptureInfo.getProtocol();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#isAudioEnabled()
	 */
	public boolean isAudioEnabled() {
		return audioCaptureInfo.isEnabled();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioCaching(int)
	 */
	public void setAudioCaching(int caching) {
		audioCaptureInfo.setCaching(caching);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioDevice(java.lang.String)
	 */
	public void setAudioDevice(String device) {
		audioCaptureInfo.setDevice(device);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioEnabled(boolean)
	 */
	public void setAudioEnabled(boolean enabled) {
		audioCaptureInfo.setEnabled(enabled);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioProtocol(java.lang.String)
	 */
	public void setAudioProtocol(String protocol) {
		audioCaptureInfo.setProtocol(protocol);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioBitrate()
	 */
	public int getAudioBitrate() {
		return audioTranscoding.getBitrate();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioChannelSize()
	 */
	public int getAudioChannelSize() {
		return audioTranscoding.getChannelSize();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioCodec()
	 */
	public String getAudioCodec() {
		return audioTranscoding.getCodec();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getAudioSampleRate()
	 */
	public int getAudioSampleRate() {
		return audioTranscoding.getSampleRate();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioBitrate(int)
	 */
	public void setAudioBitrate(int bitrate) {
		audioTranscoding.setBitrate(bitrate);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioChannelSize(int)
	 */
	public void setAudioChannelSize(int channelSize) {
		audioTranscoding.setChannelSize(channelSize);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioCodec(java.lang.String)
	 */
	public void setAudioCodec(String codec) {
		audioTranscoding.setCodec(codec);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setAudioSampleRate(int)
	 */
	public void setAudioSampleRate(int sampleRate) {
		audioTranscoding.setSampleRate(sampleRate);
	}
		
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getStreamCaching()
	 */
	public int getStreamCaching() {
		return streamingInfo.getCaching();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getStreamIp()
	 */
	public String getStreamIp() {
		return streamingInfo.getIp();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getStreamLocalPort()
	 */
	public int getStreamLocalPort() {
		return streamingInfo.getLocalPort();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getStreamProtocol()
	 */
	public String getStreamProtocol() {
		return streamingInfo.getProtocol();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getStreamTargetPort()
	 */
	public int getStreamTargetPort() {
		return streamingInfo.getTargetPort();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setStreamCaching(int)
	 */
	public void setStreamCaching(int caching) {
		streamingInfo.setCaching(caching);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setStreamIp(java.lang.String)
	 */
	public void setStreamIp(String ip) {
		streamingInfo.setIp(ip);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setStreamLocalPort(int)
	 */
	public void setStreamLocalPort(int port) {
		streamingInfo.setLocalPort(port);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setStreamProtocol(java.lang.String)
	 */
	public void setStreamProtocol(String protocol) {
		streamingInfo.setProtocol(protocol);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setStreamTargetPort(int)
	 */
	public void setStreamTargetPort(int targetPort) {
		streamingInfo.setTargetPort(targetPort);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoCaching()
	 */
	public int getVideoCaching() {
		return videoCaptureInfo.getCaching();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoDevice()
	 */
	public String getVideoDevice() {
		return videoCaptureInfo.getDevice();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoProtocol()
	 */
	public String getVideoProtocol() {
		return videoCaptureInfo.getProtocol();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#isVideoEnabled()
	 */
	public boolean isVideoEnabled() {
		return videoCaptureInfo.isEnabled();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoCaching(int)
	 */
	public void setVideoCaching(int caching) {
		videoCaptureInfo.setCaching(caching);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoDevice(java.lang.String)
	 */
	public void setVideoDevice(String device) {
		videoCaptureInfo.setDevice(device);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoEnabled(boolean)
	 */
	public void setVideoEnabled(boolean enabled) {
		videoCaptureInfo.setEnabled(enabled);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoProtocol(java.lang.String)
	 */
	public void setVideoProtocol(String protocol) {
		videoCaptureInfo.setProtocol(protocol);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoBitrate()
	 */
	public int getVideoBitrate() {
		return videoTranscoding.getBitrate();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoCodec()
	 */
	public String getVideoCodec() {
		return videoTranscoding.getCodec();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoFrameRate()
	 */
	public int getVideoFrameRate() {
		return videoTranscoding.getFrameRate();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoScale()
	 */
	public float getVideoScale() {
		return videoTranscoding.getScale();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getVideoSize()
	 */
	public Dimension getVideoSize() {
		return videoTranscoding.getSize();
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoBitrate(int)
	 */
	public void setVideoBitrate(int bitrate) {
		videoTranscoding.setBitrate(bitrate);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoCodec(java.lang.String)
	 */
	public void setVideoCodec(String codec) {
		videoTranscoding.setCodec(codec);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoFrameRate(int)
	 */
	public void setVideoFrameRate(int frameRate) {
		videoTranscoding.setFrameRate(frameRate);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoScale(float)
	 */
	public void setVideoScale(float scale) {
		videoTranscoding.setScale(scale);
	}
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setVideoSize(java.awt.Dimension)
	 */
	public void setVideoSize(Dimension size) {
		videoTranscoding.setSize(size);
	}

	@Override
	public void notifyObserver() {
		for(int i = 0; i<listener.size(); i++)
		{
			IConfigurationChangeListener observer = listener.get(i);
			observer.configurationChanged(this);
		}
		
	}

	@Override
	public void registerObserver(IConfigurationChangeListener observer) {
		listener.add(observer);
		
	}

	@Override
	public void removeObserver(IConfigurationChangeListener observer) {
		int i = listener.indexOf(observer);
		if(i>0)
			listener.remove(i);
		
	}
	
	public void transfereValues(IConfiguration config)
	{
		setAudioBitrate(config.getAudioBitrate());
		setAudioCaching(config.getAudioCaching());
		setAudioChannelSize(config.getAudioChannelSize());
		setAudioCodec(config.getAudioCodec());
		setAudioDevice(config.getVideoDevice());
		setAudioEnabled(config.isAudioEnabled());
		setAudioProtocol(config.getAudioProtocol());
		setAudioSampleRate(config.getAudioSampleRate());
		setStreamCaching(config.getStreamCaching());
		setStreamIp(config.getStreamIp());
		setStreamLocalPort(config.getStreamLocalPort());
		setStreamProtocol(config.getStreamProtocol());
		setStreamTargetPort(config.getStreamTargetPort());
		setVideoBitrate(config.getVideoBitrate());
		setVideoCaching(config.getVideoCaching());
		setVideoCodec(config.getVideoCodec());
		setVideoDevice(config.getVideoDevice());
		setVideoEnabled(config.isVideoEnabled());
		setVideoFrameRate(config.getVideoFrameRate());
		setVideoProtocol(config.getVideoProtocol());
		setVideoScale(config.getVideoScale());
		setVideoSize(config.getVideoSize());
	}

	/**
	 * @return
	 * @see de.clab.osami.conferencing.vlc.config.items.StreamingInfo#getLocalIp()
	 */
	public String getLocalIp() {
		return streamingInfo.getLocalIp();
	}

	/**
	 * @param localIp
	 * @see de.clab.osami.conferencing.vlc.config.items.StreamingInfo#setLocalIp(java.lang.String)
	 */
	public void setLocalIp(String localIp) {
		streamingInfo.setLocalIp(localIp);
	}

	@Override
	public void setVideoLocalEnabled(Boolean flag) {
		this.videoLocalEnabled = flag;
		
	}

	@Override
	public void setVideoRemoteEnabled(Boolean flag) {
		this.videoRemoteEnabled = flag;
		
	}

	/**
	 * @return the videoRemoteEnabled
	 */
	public boolean isVideoRemoteEnabled() {
		return videoRemoteEnabled;
	}

	/**
	 * @return the videoLocalEnabled
	 */
	public boolean isVideoLocalEnabled() {
		return videoLocalEnabled;
	}

}
