package de.clab.osami.conferencing.vlc.config;

import java.io.File;

public interface IStorageManager {
	
	public static final String LINUX_PATH = "config"+File.separator+"LinuxConfiguration.properties";
	public static final String WIN_PATH = "config"+File.separator+"WindowsConfiguration.properties";

	/**
	 * Stores a given configuration to a default path.
	 * @param configuration is to be stored into the file
	 */
	public void saveConfiguration(IConfiguration config);
	
	/**
	 * Loads a configuration from a default path.
	 * @return configuration Object created from the configuration file
	 */
	public IConfiguration loadConfiguration();

	/**
	 * loads configuration from a given path. The path will become default for 
	 * other operations.
	 * @param path
	 * @return IConfiguration 
	 */
	public IConfiguration loadConfiguration(String path);

	/**
	 * saves a configuration to a given path. The path will become default 
	 * for other operations. 
	 * @param config
	 * @param path
	 */
	public void saveConfiguration(IConfiguration config, String path);

	/**
	 * sets the default path to store configurations. If the parameter is null, the
	 * origin default path will be set.
	 * @param path
	 * @return String the new default path
	 */
	public String setDefaultPath(String path);

	/**
	 * Returns the current default path
	 * @return
	 */
	public String getDefaultPath();

	/**
	 * resets the default path to the origin default path.
	 * @return String default path
	 */
	public String setDefaultPath();
	
}
