package de.clab.osami.conferencing.vlc.config;

import java.awt.Dimension;

public interface IVerificator {
	
	
	/**
	 * @param audioBitRate value that is going to be verified
	 * @return null if the audioBitRate is between [8,512] 
	 * 		   otherwise a failure message
	 */
	public String verifyAudioBitrate(int audioBitRate);
	

	/**
	 * @param audioChannelSize value that is going to be verified
	 * @return null if the audioChannelSize is between [1,10] 
	 * 		   otherwise a failure message
	 */
	public String verifyAudioChannelSize(int audioChannelSize);
	
	
	/**
	 * @param audioCodec value that is going to be verified
	 * @return null if the audioCodec is one of the following 
	 * 		   {"mpga","mp3","mp4a","a52","vorb","flac","spx","s16l","wma2"} 
	 * 		   otherwise a failure message
	 */
	public String verifyAudioCodec(String audioCodec);
	
	
	/**
	 * @param audioCaching value that is going to be verified
	 * @return null if the audioCaching is equal to or greater than 0 
	 * 		   otherwise a failure message
	 */
	public String verifyAudioCaching(int audioCaching);
	
	
	/**
	 * @param audioDevice value that is going to be verified
	 * @return TODO: comment
	 */
	public String verifyAudioDevice(String audioDevice);
	
	
	/**
	 * @param audioProtocol value that is going to be verified
	 * @return TODO: comment
	 */
	public String verifyAudioProtocol(String audioProtocol);
	
	
	/**
	 * @param audioSampleRate value that is going to be verified
	 * @return null if the audioSampleRate is one of the following 
	 * 		   {8000,11025,22050,44100,48000} 
	 * 		   otherwise a failure message
	 */
	public String verifyAudioSampleRate(int audioSampleRate);
	
	
	/**
	 * @param videoBitrate value that is going to be verified
	 * @return null if the videoBitrate is between [0,32768] 
	 * 		   otherwise a failure message
	 */
	public String verifyVideoBitrate(int videoBitrate);
	
	
	/**
	 * @param videoCaching value that is going to be verified
	 * @return null if the videoCaching is equal to or greater than 0 
	 * 		   otherwise a failure message
	 */
	public String verifyVideoCaching(int videoCaching);
	
	
	/**
	 * @param videoCodec value that is going to be verified
	 * @return null if the videoCodec is one of the following 
	 * 		   {"mp1v","mp2v","mp4v","DIV1","DIV2","DIV3","H263",
				"h264","VP80","WMV1","WMV2","MJPG","theo","drac"} 
	 * 		    otherwise a failure message
	 */
	public String verifyVideoCodec(String videoCodec);
	
	
	/**
	 * @param videoDevice value that is going to be verified
	 * @return TODO: comment
	 */
	public String verifyVideoDevice(String videoDevice);
	
	
	/**
	 * @param videoFrameRate value that is going to be verified
	 * @return null if the videoFrameRate is between [0.00,99.99]
	 *         otherwise a failure message
	 */
	public String verifyVideoFrameRate(float videoFrameRate);
	
	
	/**
	 * @param videoProtocol value which is going to be verified
	 * @return TODO: comment
	 */
	public String verifyVideoProtocol(String videoProtocol);
	
	/**
	 * @param videoScale value that is going to be verified
	 * @return null if the videoScale is one of the following: 
	 * 		   {0.25,0.5,0.75,1.0,1.25,1.5,1.75,2.0}
	 *         otherwise a failure message
	 */
	public String verifyVideoScale(float videoScale);
	
	
	/**
	 * @param videoSize value that is going to be verified
	 * @return null if the videoSize is under the current screen resolution
	 *         otherwise a failure message
	 */
	public String verifyVideoSize(Dimension videoSize);
	
	
	/**
	 * @param streamCaching value that is going to be verified
	 * @return null if the streamCaching is equal to or greater than 0 
	 * 		   otherwise a failure message
	 */
	public String verifyStreamCaching(int streamCaching);
	
	
	/**
	 * @param streamIp value that is going to be verified
	 * @return null if the streamIp contains 4 integers delimited by a dot,
	 *         each of which is between [0,255]
	 *         otherwise a failure message
	 */
	public String verifyStreamIp(String streamIp);
	
	
	/**
	 * @param streamLocalPort value that is going to be verified
	 * @return null if the streamLocalPort is between [0,65535] 
	 * 		   otherwise a failure message
	 */
	public String verifyStreamLocalPort(int streamLocalPort);
	
	
	/**
	 * @param streamProtocol value that is going to be verified
	 * @return null if the streamProtocol is RTP
	 * 		   otherwise a failure message
	 */
	public String verifyStreamProtocol(String streamProtocol);
	
	
	/**
	 * @param streamTargetPort value that is going to be verified
	 * @return null if the streamTargetPort is between [0,65535] 
	 * 		   otherwise a failure message
	 */
	public String verifyStreamTargetPort(int streamTargetPort);


	public String verifyConfig(IConfiguration configuration);
	
}
