package de.clab.osami.conferencing.vlc.config;

import java.awt.Dimension;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import de.clab.osami.conferencing.vlc.util.Util;


/**TODO: check if reading from the file and writing into the file works if it is started 
		as a bundle and not as JUNIT 
		IFILE may be used instead of FILE
*/

/**
 * @author tamasmiklossy
 *
 */
public class StorageManager implements IStorageManager{
	
	//static reference to the only one object which is allowed to exist
	private static IStorageManager instance;
	//file refering to the current Configuration
	String configurationPropertyFile;
	String userFile;
	
	//TODO handle case : folder config not exists
	/**
	 * Constructor. Creates StorageManagerObject
	 */
	private StorageManager(){

		
		configurationPropertyFile = getOsDefaultPath();
	}
	
	
	/**
	 *  Gets the os dependend default path
	 * @return String path
	 */
	String getOsDefaultPath()
	{
		String filePath = "";
		if(Util.isWindows())
			filePath = WIN_PATH;
		if(Util.isUnix())
			filePath = LINUX_PATH;
		
		return filePath;
	}
	
	public static IStorageManager getInstance() {
		if (null == instance) {
			instance = new StorageManager();
		}
		return instance;
	}
	

	
	/**
	 * loads a property object from a given path
	 * @param path
	 * @return properties
	 */
	private Properties loadProperties(String path)
	{
		Properties properties = new Properties();
		
		try {
			InputStream in = new FileInputStream(path);
			properties.load(in);
			in.close();
		} catch (Exception e){
			return null;
		}
		return properties;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#loadConfiguration(java.lang.String)
	 */
	public IConfiguration loadConfiguration(String path)
	{
		userFile = path;
		Properties properties = loadProperties(userFile);
		IConfiguration config = getConfiguration(properties);
		return config;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#loadConfiguration()
	 */
	public IConfiguration loadConfiguration() {
		String file;
		if(userFile==null)
			file = configurationPropertyFile;
		else file = userFile;
		
		Properties properties = loadProperties(file);
		IConfiguration config = getConfiguration(properties);
		
		return config;
	
		//return new VlcVerificator().verifyConfig(configuration) == null ? configuration : null;
	}
	
	/**
	 * Get a configuration object from properties
	 * @param properties
	 * @return
	 */
	private IConfiguration getConfiguration(Properties properties)
	{
		IConfiguration configuration = Configuration.getInstance();

		
		try{
			configuration.setAudioBitrate(Integer.parseInt(properties.getProperty("audioBitRate")));
			configuration.setAudioChannelSize(Integer.parseInt(properties.getProperty("audioChannelSize")));
			configuration.setAudioCaching(Integer.parseInt(properties.getProperty("audioCaching")));
			configuration.setAudioCodec(properties.getProperty("audioCodec"));
			configuration.setAudioDevice(properties.getProperty("audioDevice"));
			configuration.setAudioEnabled(Boolean.parseBoolean(properties.getProperty("audioEnabled")));
			configuration.setAudioProtocol(properties.getProperty("audioProtocol"));
			configuration.setAudioSampleRate(Integer.parseInt(properties.getProperty("audioSampleRate")));
			configuration.setVideoBitrate(Integer.parseInt(properties.getProperty("videoBitrate")));
			configuration.setVideoCaching(Integer.parseInt(properties.getProperty("videoCaching")));
			configuration.setVideoCodec(properties.getProperty("videoCodec"));
			configuration.setVideoDevice(properties.getProperty("videoDevice"));
			configuration.setVideoEnabled(Boolean.parseBoolean(properties.getProperty("videoEnabled")));
			configuration.setVideoFrameRate(Integer.parseInt(properties.getProperty("videoFrameRate")));
			configuration.setVideoProtocol(properties.getProperty("videoProtocol"));
			configuration.setVideoScale(Float.parseFloat(properties.getProperty("videoScale")));
		
			String videoSize = properties.getProperty("videoSize");
			String width = videoSize.substring(0,videoSize.indexOf(" "));
			String heigh = videoSize.substring(videoSize.indexOf(" ")+1);
			configuration.setVideoSize(new Dimension(Integer.parseInt(width),Integer.parseInt(heigh)));
				
			configuration.setStreamCaching(Integer.parseInt(properties.getProperty("streamCaching")));
			configuration.setStreamIp(properties.getProperty("ip"));
			configuration.setStreamLocalPort(Integer.parseInt(properties.getProperty("localPort")));
			configuration.setStreamProtocol(properties.getProperty("streamProtocol"));
			configuration.setStreamTargetPort(Integer.parseInt(properties.getProperty("targetPort")));
		
		}catch( Exception e ){
			return null;
		}
		
		return configuration;
	}
	

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#saveConfiguration(de.clab.osami.conferencing.vlc.config.IConfiguration)
	 */
	public void saveConfiguration(IConfiguration configuration) 
	{
		String file;
		if(userFile!=null)
			file = userFile;
		else file = configurationPropertyFile;

		Properties properties = createProperty(configuration);
		writeToFile(file, properties);
		

	}
	
	/**
	 * creates a properties object specified by a configuration
	 * @param configuration
	 * @return Properties
	 */
	private Properties createProperty(IConfiguration configuration)
	{
		Properties properties = new Properties();
		
		//copy the values from configuration object into properties object
		properties.setProperty("audioBitRate",      Integer.toString(configuration.getAudioBitrate()));
		properties.setProperty("audioChannelSize" , Integer.toString(configuration.getAudioChannelSize()));
		properties.setProperty("audioCaching",		Integer.toString(configuration.getAudioCaching()));
		properties.setProperty("audioCodec",		configuration.getAudioCodec());
		properties.setProperty("audioDevice",		configuration.getAudioDevice());
		properties.setProperty("audioEnabled",		Boolean.toString(configuration.isAudioEnabled()));
		properties.setProperty("audioProtocol",		configuration.getAudioProtocol());
		properties.setProperty("audioSampleRate",	Integer.toString(configuration.getAudioSampleRate()));
		properties.setProperty("videoBitrate",		Integer.toString(configuration.getVideoBitrate()));
		properties.setProperty("videoCaching",		Integer.toString(configuration.getVideoCaching()));
		properties.setProperty("videoCodec",		configuration.getVideoCodec());
		properties.setProperty("videoDevice",		configuration.getVideoDevice());
		properties.setProperty("videoEnabled",		Boolean.toString(configuration.isVideoEnabled()));
		properties.setProperty("videoFrameRate",	Integer.toString(configuration.getVideoFrameRate()));
		properties.setProperty("videoProtocol",		configuration.getVideoProtocol());
		properties.setProperty("videoScale",		Float.toString(configuration.getVideoScale()));
		properties.setProperty("videoSize",			configuration.getVideoSize().width+" "+configuration.getVideoSize().height);
		properties.setProperty("streamCaching",		Integer.toString(configuration.getStreamCaching()));
		properties.setProperty("ip",				configuration.getStreamIp());
		properties.setProperty("localPort",			Integer.toString(configuration.getStreamLocalPort()));
		properties.setProperty("streamProtocol",	configuration.getStreamProtocol());
		properties.setProperty("targetPort",		Integer.toString(configuration.getStreamTargetPort()));
		return properties;
	}
	
	/**
	 * writes properties to disk, specified by a given file path
	 * @param path
	 * @param properties
	 */
	private void writeToFile(String path, Properties properties)
	{
		//store the properties object into the file
		try {
			OutputStream out = new FileOutputStream(path);
			properties.store(out, "Current Configuration of VideoConference");
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		} 
	}
	
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#saveConfiguration(de.clab.osami.conferencing.vlc.config.IConfiguration, java.lang.String)
	 */
	public void saveConfiguration(IConfiguration config, String path)
	{
		this.userFile = path;
		Properties properties = createProperty(config);
		writeToFile(this.userFile,properties);
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#setDefaultPath(java.lang.String)
	 */
	public String setDefaultPath(String path)
	{
		userFile = null;
		if(path!=null)
			configurationPropertyFile = path;
		else configurationPropertyFile = getOsDefaultPath();
		return configurationPropertyFile;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#setDefaultPath()
	 */
	public String setDefaultPath()
	{
		configurationPropertyFile = getOsDefaultPath();
		return configurationPropertyFile;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IStorageManager#getDefaultPath()
	 */
	public String getDefaultPath()
	{
		return configurationPropertyFile;
	}
}