package de.clab.osami.conferencing.vlc.processor;

import java.awt.Dimension;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.config.IConfiguration;

public class V4l2Configurator implements IConfigurator {

	private final static Logger log = Logger.getLogger("vc-auto-config");
	private static final int audioBitrate = 128;
	private static final int audioChannelSize = 2;
	private static final String audioCodec = "mpga";
	private static final String audioDevice = "plughw:1,0";
	private static final boolean audioEnabled = true;
	private static final String audioProtocol = "alsa";
	private static final int audioSampleRate = 8000;
	private static final int videoBitrate = 500;
	private static final int videoCaching = 100;
	private static final String videoCodec = "DIV3";
	private static final String videoDevice = "/dev/video0";
	private static final boolean videoEnabled = true;
	private static final int videoFrameRate = 30;
	private static final String videoProtocol = "v4l2";
	private static final float videoScale = 1;
	private static final Dimension videoSize = new Dimension(352,288);
	private static final int streamCaching = 100;
	private static final String ip = "127.0.0.1";
	private static final int localPort = 5004;
	private static final String streamProtocol = "rtp";
	private static final int targetPort = 5004;
	private static final String localIp = "127.0.0.1";
	private static final boolean localVideoEnabled = true;
	private static final boolean remoteVideoEnabled = true;
  
	@Override
	public void autoConfig(IConfiguration config) {
		config.setAudioBitrate(audioBitrate);
		config.setAudioChannelSize(audioChannelSize);
		config.setAudioCodec(audioCodec);
		config.setAudioDevice(findAudioDevice());
		config.setAudioEnabled(audioEnabled);
		config.setAudioProtocol(findAudioProtocol());
		config.setAudioSampleRate(audioSampleRate);
		
		config.setVideoBitrate(videoBitrate);
		config.setVideoCaching(videoCaching);
		config.setVideoCodec(videoCodec);
		config.setVideoDevice(findVideoDevice());
		config.setVideoEnabled(videoEnabled);
		config.setVideoFrameRate(videoFrameRate);
		config.setVideoProtocol(findVideoProtocol());
		config.setVideoScale(videoScale);
		config.setVideoSize(videoSize);
		
		config.setStreamCaching(streamCaching);
		config.setStreamIp(getLocalIp());
		config.setStreamLocalPort(localPort);
		config.setStreamProtocol(streamProtocol);
		config.setStreamTargetPort(targetPort);
		
		config.setVideoLocalEnabled(localVideoEnabled);
		config.setVideoRemoteEnabled(remoteVideoEnabled);
		config.setLocalIp(getLocalIp());
	}
	
	private String findAudioDevice()
	{
		//TODO find audio devices
		log.info("Searching for audio devices");
		String result = getAudiodevice();
		log.info("Audio device ist set to "+result);
		return result;
	}
	
	private String findVideoDevice()
	{
		//TODO find video devices
		log.info("Searching for video devices");
		String result = getVideodevice();
		log.info("Video device ist set to "+result);
		return result;
	}
	
	private String getLocalIp()
	{
		log.info("Local ip adress:");
		String ip = getIp();
		try {
			ip = InetAddress.getLocalHost().getHostAddress();
		} catch (UnknownHostException e) {
			log.error("couldn't find out local ip adress");
		}
		log.info(ip);
		return ip;
	}
	
	private String findAudioProtocol()
	{
		log.info("Searching audio protocol");
		//TODO audio protocol detection
		String result = getAudioprotocol();
		log.info("Audio protocol ist set to "+result);
		return result;
	}
	
	private String findVideoProtocol()
	{
		log.info("Searching video protocol");
		//TODO video protocol detection
		String result = getVideoprotocol();
		log.info("Video protocol ist set to "+result);
		return result;
	}

	/**
	 * @return the ip
	 */
	private static String getIp() {
		return ip;
	}

	/**
	 * @return the audiodevice
	 */
	private static String getAudiodevice() {
		return audioDevice;
	}

	/**
	 * @return the videodevice
	 */
	private static String getVideodevice() {
		return videoDevice;
	}

	/**
	 * @return the audioprotocol
	 */
	private static String getAudioprotocol() {
		return audioProtocol;
	}

	/**
	 * @return the videoprotocol
	 */
	private static String getVideoprotocol() {
		return videoProtocol;
	}

}
