package de.clab.osami.conferencing.vlc.processor;

import java.awt.Dimension;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.config.IConfiguration;
import de.clab.osami.conferencing.vlc.util.Util;

class V4l2ParameterExtractor extends IParameterExtractor{

	private final static Logger log = Logger.getLogger(V4l2ParameterExtractor.class);
	
	V4l2ParameterExtractor(IConfiguration config) {
		super(config);
	}

	@Override
	String getClientLocator() {
		String protocol = getConfig().getStreamProtocol();
		String ip = getConfig().getLocalIp();
		int port = getConfig().getStreamLocalPort();
		String locator = protocol+"://"+ip+":"+port;
		log.info("Locator is set to "+locator);
		return locator;
	}

	@Override
	String[] getClientOptions() {
		ArrayList<String> list = new ArrayList<String>();
		
		// construct caching option
		String protocol = getConfig().getStreamProtocol();
		int cach = getConfig().getStreamCaching();
		String caching = ":"+protocol+"-"+"caching="+cach;
		
		
		//TODO only caching options considered
		list.add(caching);
		

//		list.add(":no-overlay");
	//	list.add("");
		log.info("client options:");
		for(int i = 0; i<list.size();i++)
			log.info(list.get(i));		
		return Util.listToArray(list);
	}

	@Override
	String getServerLocator() {
		String locator = "v4l2://";
		return locator;
	}

	@Override
	
	String[] getServerOptions() {
		ArrayList<String> list = new ArrayList<String>();
			
		//caching
		int vCaching = getConfig().getVideoCaching();
		
		// default transcoding
		String audioTranscoding = "acodec=none";
		String videoTranscoding = "vcodec=none";
		
		// Streaming configuration
		String streamingProtocol = getConfig().getStreamProtocol();
		int streamingPort = getConfig().getStreamTargetPort();
		String ip = getConfig().getStreamIp();
		

		//Audio Configuration
		if(getConfig().isAudioEnabled())
		{
			String audioProtocol = getConfig().getAudioProtocol();
			String audioDevice = getConfig().getAudioDevice();
			String audioCodec = getConfig().getAudioCodec();
			int audioBitrate = getConfig().getAudioBitrate();
			int audioChannels = getConfig().getAudioChannelSize();
			int audioSampleRate = getConfig().getAudioSampleRate();
			
			audioTranscoding="acodec="+audioCodec+
									",ab="+audioBitrate+
									",channels="+audioChannels+
									",samplerate="+audioSampleRate;
			
			String audioLocator = ":input-slave="+audioProtocol+"://"+audioDevice;
			list.add(audioLocator);
			list.add(":v4l2-standard=0");
			
		}
		
		// add caching option
		String v4l2Caching = ":v4l2"+"-"+"caching="+vCaching;
		list.add(v4l2Caching);
		
		//video configuration
		if(getConfig().isVideoEnabled())
		{
			int vBitRate = getConfig().getVideoBitrate();
			String vCodec = getConfig().getVideoCodec();
			float vFrameRate = getConfig().getVideoFrameRate();
			float vScale = getConfig().getVideoScale();
			Dimension size = getConfig().getVideoSize();
			videoTranscoding = "vcodec="+vCodec
								+",vb="+vBitRate
								+",fps="+(int)vFrameRate
								+",scale="+vScale
								+",width="+size.width
								+",height="+size.height;
						
		}
		
		//sout example
		//	":sout=#transcode{vcodec=DIV3,vb=1500,fps=30,scale=1,acodec=a52,ab=128,channels=2,samplerate=8000}:duplicate{dst=rtp{dst="+ip+",port=5004,mux=ts},dst=display}

		String sout = ":sout=#transcode{"+videoTranscoding+","+audioTranscoding
										+"}:duplicate{dst="+streamingProtocol
										+"{dst="+ip+",port="+streamingPort+",mux=ts},dst=display}";
		log.debug("sout streaming message: "+sout);
		list.add(sout);
		list.add(":no-sout-rtp-sap");
		list.add(":no-sout-standard-sap");
		list.add(":sout-keep");
		

		return Util.listToArray(list);
	}



}
