package de.clab.osami.conferencing.vlc.processor;

public class VlcCodecManager implements ICodecManager {

	
	private static VlcCodecManager instance;
	
	private String[] audioNames;
	private String[] audioCodecs;
	private String[] videoNames;
	private String[] videoCodecs;
	
	private VlcCodecManager(){
		audioNames = new String[]{
				"MPEG Audio",
				"MP3",
				"MPEG 4 Audio(AAC)",
				"A52/AC-3",
				"Vorbis",
				"Flac",
				"Speex",
				"WAV",
				"WMA2"
		};
		
		audioCodecs  = new String[]{
				"mpga",
				"mp3",
				"mp4a",
				"a52",
				"vorb",
				"flac",
				"spx",
				"s16l",
				"wma2"
		};
		
		videoNames = new String[]{"MPEG-1",		
				"MPEG-2",			
				"MPEG-4",			
				"DIVX1",		
				"DIVX2",			
				"DIVX3",			
				"H-263",			
				"H-264",			
				"VP8",			
				"WMV1",		
				"WMV2",		
				"M-JPEG",		
				"Theora",		
				"Dirac"		};
		
		videoCodecs = new String[]{
				"mp1v",
				"mp2v",
				"mp4v",
				"DIV1",
				"DIV2",
				"DIV3",
				"H263",
				"h264",
				"VP80",
				"WMV1",
				"WMV2",
				"MJPG",
				"theo",
				"drac"};
	}
	
	public static VlcCodecManager getInstance() {
		if (null == instance) {
			instance = new VlcCodecManager();
		}
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.processor.ICodecManager#audioCodec2Name(java.lang.String)
	 */
	public String audioCodec2Name(String codec) {
		int i;
		for(i=0;i<audioCodecs.length && !audioCodecs[i].equals(codec);i++);
		return i==audioCodecs.length ? null : audioNames[i];
	}

	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.processor.ICodecManager#audioName2codec(java.lang.String)
	 */
	public String audioName2codec(String name) {
		int i;
		for(i=0;i<audioNames.length && !audioNames[i].equals(name);i++);
		return i==audioNames.length ? null : audioCodecs[i];
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.processor.ICodecManager#getAvailableAudioCodecs()
	 */
	public String[] getAvailableAudioCodecs() {
		//TODO cloning the object
		return audioCodecs;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.processor.ICodecManager#getAvailableVideoCodecs()
	 */
	public String[] getAvailableVideoCodecs() {
		//TODO return a cloning object
		return videoCodecs;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.processor.ICodecManager#videoCodec2Name(java.lang.String)
	 */
	public String videoCodec2Name(String codec) {
		int i;
		for(i=0;i<videoCodecs.length && !videoCodecs[i].equals(codec);i++);
		return i==videoCodecs.length ? null : videoNames[i];
	}

	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.processor.ICodecManager#videoName2codec(java.lang.String)
	 */
	public String videoName2codec(String name) {
		int i;
		for(i=0;i<videoNames.length && !videoNames[i].equals(name);i++);
		return i==videoNames.length ? null : videoCodecs[i];
	}

}
