package de.clab.osami.conferencing.vlc.processor;

import java.awt.Canvas;
import java.util.ArrayList;
import java.util.Arrays;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.conferencingSystem.IProcessor;
import de.clab.osami.conferencing.vlc.config.IConfiguration;
import de.clab.osami.conferencing.vlc.config.IVerificator;

public class VlcProcessor implements IProcessor {

	private final static Logger log = Logger.getLogger(VlcProcessor.class);
	private IConfiguration config;
	private IParameterExtractor extractor;
	private IConfigurator configurator;
	private VlcCodecManager codecManager;
	private IVerificator verificator;
	public VlcPlayer serverPlayer;
	public VlcPlayer clientPlayer;

	
	public VlcProcessor()
	{
		
		// Player
		serverPlayer = new VlcPlayer("Local video");
		clientPlayer = new VlcPlayer("remote video");
		
		// configurator
		configurator = new V4l2Configurator();
		
		
		//codec Manager
		codecManager = VlcCodecManager.getInstance();
		
		// verificator
		verificator = new VlcVerificator();
	}
	
	@Override
	public String init(IConfiguration config) {

		this.config = config;
		// map extractor to configurator
		extractor = new V4l2ParameterExtractor(config);
		config.registerObserver(this);
		//reconfig();
		return null;
	}
	
	@Override
	public ArrayList<String> getAudioCodecs() {
		return (ArrayList<String>) Arrays.asList(codecManager.getAvailableAudioCodecs());
	}

	@Override
	public ArrayList<String> getAudioDevices() {
		// TODO find audio devices
		ArrayList list = new ArrayList();
		return null;
	}

	@Override
	public ArrayList<String> getVideoCodecs() {
		return (ArrayList<String>) Arrays.asList(codecManager.getAvailableVideoCodecs());
	}

	@Override
	public ArrayList<String> getVideoDevices() {
		// TODO find videoDevices
		return null;
	}

	@Override
	public Object getRemoteVisualComponent() {
		return clientPlayer.getCanvas();
	}

	@Override
	public String start() {
		String msg = validate();
		// TODO Threading?
		configure();
		if(msg==null)
		{
			msg = serverPlayer.start(config.isVideoLocalEnabled());
			msg+= " "+clientPlayer.start(config.isVideoRemoteEnabled());
		}
		return msg;
	}

	public void configure()
	{
		log.info("Configure vlc processors");

		String clientLocator = extractor.getClientLocator();
		String[] clientOptions = extractor.getClientOptions();
		String serverLocator = extractor.getServerLocator();
		String[] serverOptions = extractor.getServerOptions();
		
		log.info("Preparing vlc client with "+clientLocator+" "+clientOptions[0]);
		clientPlayer.configure(clientLocator,clientOptions,config.getVideoSize());
		serverPlayer.configure(serverLocator,serverOptions,config.getVideoSize());	
	}
	
	@Override
	public String stop() {
		log.info("stoping vlc processors");
		clientPlayer.stop();
		serverPlayer.stop();
		return null;
	}

	@Override
	public String validate() {
		return verificator.verifyConfig(config);
		
	}

	@Override
	public Object getLocalVisualComponent() {
		return serverPlayer.getCanvas();
	}

	@Override
	public String autoConfig() {
		configurator.autoConfig(config);
		return null;
	}

	@Override
	public IConfiguration getConfiguration() {
		return config;
	}

	@Override
	public String setConfiguration(IConfiguration config) {
		this.config = config;
		return null;
	}

	@Override
	public String setLocalVisualComponent(Object component) {
		if(component instanceof Canvas)
		{
			serverPlayer.setCanvas((Canvas) component);
			return null;
		}
		return "only components of type java.awt.Panel can be set for vlc";
	}

	@Override
	public String setRemoteVisualComponent(Object component) {
		if(component instanceof Canvas)
		{
			clientPlayer.setCanvas((Canvas) component);
			return null;
		}
		return "only components of type java.awt.Panel can be set for vlc";
	}

	@Override
	public void configurationChanged(IConfiguration config) {
		log.info("Configuration changed...");
		this.config = config;
		extractor.setConfig(config);
	}


}
