package de.clab.osami.conferencing.vlc.processor;

import java.awt.Dimension;
import java.awt.Toolkit;

import de.clab.osami.conferencing.vlc.config.IConfiguration;
import de.clab.osami.conferencing.vlc.config.IVerificator;

public class VlcVerificator implements IVerificator{

	private String AUDIOBITRATE_FAILURE_MESSAGE     = "Failure: audioBitRate is only allowed to be between [8,512]!";
	private String AUDIOCHANNELSIZE_FAILURE_MESSAGE = "Failure: audioChannelSize is only allowed to be 1 (mono) or 2 (stereo)";
	private String AUDIOCODEC_FAILURE_MESSAGE		= "Failure: audioCodec is only allowed to be one of the following:mpga, mp3 mp4a a52 vorb flac spx s16l wma2!";
	private String AUDIOCACHING_FAILURE_MESSAGE		= "Failure: audioCaching is allowed to be either equal to or greater than 0!";
	private String AUDIODEVICE_FAILURE_MESSAGE		= ""; //TODO: implement
	private String AUDIOPROTOCOL_FAILURE_MESSAGE	= ""; //TODO: implement
	private String AUDIOSAMPLERATE_FAILURE_MESSAGE  = "Failure: audioSampleRate is only allowed to be one of the following: 8000,11025,22050,44100,48000!";
	private String STREAMCACHING_FAILURE_MESSAGE    = "Failure: streamCaching is only allowed to be either equal to or greater than 0!";
	private String STREAMIP_FAILURE_MESSAGE         = "Failure: streamIp should look like as [0-255].[0-255].[0-255].[0-255]!";
	private String STREAMLOCALPORT_FAILURE_MESSAGE  = "Failure: streamLocalPort is only allowed to be between [0,65535]!";
	private String STREAMPROTOCOL_FAILURE_MESSAGE   = "Failure: streamProtocol is only allowed to be RTP";
	private String STREAMTARGETPORT_FAILURE_MESSAGE = "Failure: streamTargetPort is only allowed to be between [0,65535]!";
	private String VIDEOBITRATE_FAILURE_MESSAGE     = "Failure: videoBitrate is only allowed to be between [0,32768]!";
	private String VIDEOCACHING_FAILURE_MESSAGE     = "Failure: videoCaching is only allowed to be either equal to or greater than 0!";
	private String VIDEOCODEC_FAILURE_MESSAGE       = "Failure: videoCodec is only allowed to be one of the following:\n\tmp1v,mp2v,mp4v,DIV1,DIV2,DIV3,H263,h264,P80,WMV1,WMV2,MJPG,theo,drac";
	private String VIDEODEVICE_FAILURE_MESSAGE      = ""; //TODO: implement
	private String VIDEOFRAMERATE_FAILURE_MESSAGE   = "Failure: videoFrameRate is only allowed to be between [0,99]!";
	private String VIDEOPROTOCOL_FAILURE_MESSAGE    = ""; //TODO: implement
	private String VIDEOSCALE_FAILURE_MESSAGE       = "Failure: videoScale is one of the following: 0.25,0.5,0.75,1.0,1.25,1.5,1.75,2.0!";
	private String VIDEOSIZE_FAILURE_MESSAGE        = "Failure: videoSize can not be more than the current screen resolution: ";
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioBitrate(int)
	 */
	@Override
	public String verifyAudioBitrate(int audioBitRate) {
		return 8<=audioBitRate && audioBitRate<=512 ? null : AUDIOBITRATE_FAILURE_MESSAGE; 
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioChannelSize(int)
	 */
	@Override
	public String verifyAudioChannelSize(int audioChannelSize) {
		return 1<=audioChannelSize && audioChannelSize<=2 ? null : AUDIOCHANNELSIZE_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioCodec(java.lang.String)
	 */
	@Override
	public String verifyAudioCodec(String audioCodec) {
		String[] audioCodecs = VlcCodecManager.getInstance().getAvailableAudioCodecs();
		int i;
		for(i=0;i<audioCodecs.length && !audioCodecs[i].equals(audioCodec);i++);
		return i==audioCodecs.length ?  AUDIOCODEC_FAILURE_MESSAGE: null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioCaching(int)
	 */
	@Override
	public String verifyAudioCaching(int audioCaching) {
		return 0<=audioCaching ? null : AUDIOCACHING_FAILURE_MESSAGE;
	}
	
	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioDevice(java.lang.String)
	 */
	@Override
	public String verifyAudioDevice(String audioDevice) {
		// TODO: implement, at this moment every string will be accepted as audioDevice
		return null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioProtocol(java.lang.String)
	 */
	@Override
	public String verifyAudioProtocol(String audioProtocol) {
		// TODO: implement, at this moment every string will be accepted as audioProtocol
		return null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyAudioSampleRate(int)
	 */
	@Override
	public String verifyAudioSampleRate(int audioSampleRate) {
		switch(audioSampleRate){
			case 8000  : return null;
			case 11025 : return null;
			case 22050 : return null;
			case 44100 : return null;
			case 48000 : return null;
			default    : return AUDIOSAMPLERATE_FAILURE_MESSAGE;
		}
		
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyStreamCaching(int)
	 */
	@Override
	public String verifyStreamCaching(int streamCaching) {
		return 0<=streamCaching ? null : STREAMCACHING_FAILURE_MESSAGE; 
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyStreamIp(java.lang.String)
	 */
	@Override
	public String verifyStreamIp(String streamIp) {
		if (streamIp == null) return STREAMIP_FAILURE_MESSAGE;
		
	    String[] parts = streamIp.split( "\\." );
	    if ( parts.length != 4 ){
		       return STREAMIP_FAILURE_MESSAGE;
		}

		for ( String s : parts ){
		   int i;
		   try{ 
			   i = Integer.parseInt( s );
		   }catch (NumberFormatException e) {
			   return STREAMIP_FAILURE_MESSAGE;
		   }
		   if ( (i < 0) || (i > 255) ){
		        return STREAMIP_FAILURE_MESSAGE; 
	 	   }
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyStreamLocalPort(int)
	 */
	@Override
	public String verifyStreamLocalPort(int streamLocalPort) {
		return 0<=streamLocalPort && streamLocalPort<=65535 ? null : STREAMLOCALPORT_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyStreamProtocol(java.lang.String)
	 */
	@Override
	public String verifyStreamProtocol(String streamProtocol) {
		return streamProtocol != null && streamProtocol.equals("rtp") ? null : STREAMPROTOCOL_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyStreamTargetPort(int)
	 */
	@Override
	public String verifyStreamTargetPort(int streamTargetPort) {
		return 0<=streamTargetPort && streamTargetPort<=65535 ? null : STREAMTARGETPORT_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoBitrate(int)
	 */
	@Override
	public String verifyVideoBitrate(int videoBitrate) {
		return 0<=videoBitrate && videoBitrate<=32768 ? null : VIDEOBITRATE_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoCaching(int)
	 */
	@Override
	public String verifyVideoCaching(int videoCaching) {
		return 0<=videoCaching ? null : VIDEOCACHING_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoCodec(java.lang.String)
	 */
	@Override
	public String verifyVideoCodec(String videoCodec) {
		String[] videoCodecs = VlcCodecManager.getInstance().getAvailableVideoCodecs();
		int i;
		for(i=0;i<videoCodecs.length && !videoCodecs[i].equals(videoCodec);i++);
		return i==videoCodecs.length ? VIDEOCODEC_FAILURE_MESSAGE : null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoDevice(java.lang.String)
	 */
	@Override
	public String verifyVideoDevice(String videoDevice) {
		// TODO implement, at this moment every String will be accepted as videoDevice
		return null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoFrameRate(float)
	 */
	@Override
	public String verifyVideoFrameRate(float videoFrameRate) {
		return 0<=videoFrameRate && videoFrameRate<=99 ? null : VIDEOFRAMERATE_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoProtocol(java.lang.String)
	 */
	@Override
	public String verifyVideoProtocol(String videoProtocol) {
		// TODO implement, at this moment every String will be accepted as videoProtocol
		return null;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoScale(float)
	 */
	@Override
	public String verifyVideoScale(float videoScale) {
		if( videoScale == 0.25 ||
			videoScale == 0.5  ||
			videoScale == 0.75 ||
			videoScale == 1.0  ||
			videoScale == 1.25 ||
			videoScale == 1.5  ||
			videoScale == 1.75 ||
			videoScale == 2.0)  return null;
		return VIDEOSCALE_FAILURE_MESSAGE;
	}

	/* (non-Javadoc)
	 * @see de.clab.osami.conferencing.vlc.config.IVerificator#verifyVideoSize(java.awt.Dimension)
	 */
	@Override
	public String verifyVideoSize(Dimension videoSize) {
		//get the current resolution of the screen
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		return 0<=videoSize.height && 
				  videoSize.height<=screenSize.height && 
		       0<=videoSize.width && 
				  videoSize.width<=screenSize.width ? null : VIDEOSIZE_FAILURE_MESSAGE+screenSize.width+"x"+screenSize.height;
	}

	public String verifyConfig(IConfiguration configuration){
		return verifyAudioBitrate(configuration.getAudioBitrate()) == null &&  
			   verifyAudioChannelSize(configuration.getAudioChannelSize()) == null &&
			   verifyAudioCaching(configuration.getAudioCaching()) == null &&
			   verifyAudioCodec(configuration.getAudioCodec())== null &&
			   verifyAudioDevice(configuration.getAudioDevice()) == null &&
			   verifyAudioProtocol(configuration.getAudioProtocol()) == null &&
			   verifyAudioSampleRate(configuration.getAudioSampleRate()) == null &&
			   verifyVideoBitrate(configuration.getVideoBitrate()) == null &&
			   verifyVideoCaching(configuration.getVideoCaching()) == null &&
			   verifyVideoCodec(configuration.getVideoCodec()) == null &&
			   verifyVideoDevice(configuration.getVideoDevice()) == null &&
			   verifyVideoFrameRate(configuration.getVideoFrameRate()) == null &&
			   verifyVideoProtocol(configuration.getVideoProtocol()) == null &&
			   verifyVideoScale(configuration.getVideoScale()) == null &&
			   verifyVideoSize(configuration.getVideoSize()) == null &&
			   verifyStreamCaching(configuration.getStreamCaching()) == null &&
			   verifyStreamIp(configuration.getStreamIp()) == null &&
			   verifyStreamLocalPort(configuration.getStreamLocalPort()) == null &&
			   verifyStreamProtocol(configuration.getStreamProtocol()) == null &&
			   verifyStreamTargetPort(configuration.getStreamTargetPort()) == null 
		 ? null : "Verification of configuration failed";
	}

}

