package de.clab.osami.conferencing.vlc.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Panel;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;

import de.clab.osami.conferencing.vlc.processor.VlcCodecManager;

public class AudioPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private JSpinner audioCachingSpinner;
	private JSpinner audioBitRateSpinner;
	private JComboBox audioCodecBox;
	private JTextField audioDeviceTextField;
	private JTextField audioProtocolTextField;
	private JCheckBox audioEnabledCheckBox;
	private JComboBox audioSampleRateBox;
	private JComboBox audioChannelSizeBox;

	public AudioPanel(Panel parent){
		
		parent.add(this,BorderLayout.NORTH);
		setLayout(new GridLayout(8,2,0,5));
		setBorder(new TitledBorder("Audio"));
		
		audioEnabledCheckBox = new JCheckBox();
		addGUIElement("AudioEnabled:",audioEnabledCheckBox);
		
		audioCachingSpinner = new JSpinner(new SpinnerNumberModel(0,0,Integer.MAX_VALUE,1));
		addGUIElement("AudioCaching:",    audioCachingSpinner);
		
		audioBitRateSpinner = new JSpinner(new SpinnerNumberModel(8,8,512,1));
		addGUIElement("AudioBitRate:",    audioBitRateSpinner);
		
		audioCodecBox = new JComboBox(VlcCodecManager.getInstance().getAvailableAudioCodecs());
		addGUIElement("AudioCodec:"  ,    audioCodecBox);
		
		audioDeviceTextField = new JTextField();
		addGUIElement("AudioDevice:" ,    audioDeviceTextField);
		
		audioProtocolTextField = new JTextField();
		addGUIElement("AudioProtocol:",   audioProtocolTextField);
		
		audioSampleRateBox = new JComboBox(new Integer[]{8000,11025,22050,44100,48000});
		addGUIElement("AudioSampleRate:", audioSampleRateBox);
		
		audioChannelSizeBox = new JComboBox(new Integer[]{1,2});
		addGUIElement("AudioChannelSize:",audioChannelSizeBox);
	}
	
	private void addGUIElement(String text, Component component){
		new GUIElement(this,text,component);
	}

	/**
	 * enable/disable if the audioEnabled checkBox has been selected/deselected
	 */
	@Override
	public void setEnabled(boolean isEnabled){
		/**enable all the child elements of this audiopanel except the first two (they are representing the checkbox itself*/
		Component[] components = getComponents();
		for(int i=2;i<components.length;i++)
			components[i].setEnabled(isEnabled);
	}
	
	//GETTERs
	public JSpinner getAudioCachingSpinner() {
		return audioCachingSpinner;
	}

	public JSpinner getAudioBitRateSpinner() {
		return audioBitRateSpinner;
	}

	public JComboBox getAudioCodecBox() {
		return audioCodecBox;
	}

	public JTextField getAudioDeviceTextField() {
		return audioDeviceTextField;
	}

	public JTextField getAudioProtocolTextField() {
		return audioProtocolTextField;
	}

	public JCheckBox getAudioEnabledCheckBox() {
		return audioEnabledCheckBox;
	}

	public JComboBox getAudioSampleRateBox() {
		return audioSampleRateBox;
	}

	public JComboBox getAudioChannelSizeBox() {
		return audioChannelSizeBox;
	}
	
	
}
