package de.clab.osami.conferencing.vlc.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.config.IConfiguration;
import de.clab.osami.conferencing.vlc.config.IVerificator;
import de.clab.osami.conferencing.vlc.config.StorageManager;
import de.clab.osami.conferencing.vlc.processor.IConfigurator;
import de.clab.osami.conferencing.vlc.processor.V4l2Configurator;
import de.clab.osami.conferencing.vlc.processor.VlcVerificator;

public class ConfigurationPanel extends Panel implements ActionListener, ChangeListener{
	/**
	 *  
	 */
	private static final long serialVersionUID = 1L;
	
	private IConfiguration config;
	
	//GUI Elements representing audio data
	private JSpinner   audioCachingSpinner;
	private JSpinner   audioBitRateSpinner;
	private JComboBox  audioCodecBox;
	private JTextField audioDeviceTextField;
	private JTextField audioProtocolTextField;
	private JCheckBox  audioEnabledCheckBox;
	private JComboBox  audioSampleRateBox;
	private JComboBox  audioChannelSizeBox;
	
	//GUI Elements representing video data
	private JSpinner   videoCachingSpinner;
	private JSpinner   videoBitRateSpinner;
	private JComboBox  videoCodecBox;
	private JTextField videoDeviceTextField;
	private JTextField videoProtocolTextField;
	private JCheckBox  videoEnabledCheckBox;
	private JSpinner   videoFrameRateSpinner;
	private JSpinner   videoSizeWidthSpinner;
	private JSpinner   videoSizeHeightSpinner;
	private JComboBox  videoScaleBox;
	
	//GUI Elements representing streaming data
	private JSpinner   streamCachingSpinner;
	private JTextField streamIPTextField;
	private JComboBox  streamProtocolBox;
	private JSpinner   streamLocalPortSpinner;
	private JSpinner   streamTargetPortSpinner;

	//BUTTONs
	private JButton loadButton;
	private JButton saveButton;
	private JButton saveAsButton;
	private JButton resetButton;


	//Panels needed because of the ENABLE Checkbox
	private AudioPanel audioPanel;
	private VideoPanel videoPanel;
	
	private String errorMessageSum;
	private String confirmMessage;
	
	private JFileChooser fileChooser;
	
	private final static Logger log = Logger.getLogger(ConfigurationPanel.class);
	
	private boolean initFlag;

	
	public ConfigurationPanel(IConfiguration config){
		initFlag = true;
		this.config = config;
		
		createGUI();
		if(config != null) {
			showConfigurationsData();
		}
		initFlag = false;
		
		//create File Chooser
		fileChooser = new JFileChooser();
		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.setFileFilter(new FileFilter(){ //only files with the extension .properties are allowed
			@Override
			public boolean accept(File f) {
				return f.getName().endsWith(".properties");
			}
			@Override
			public String getDescription() {
				// TODO Auto-generated method stub
				return ".properties";
			}});
	}

	/**
	 * creates the GUI elements and assign listeners to the buttons and checkboxes
	 */
	private void createGUI(){
		
		setLayout(new GridLayout(1,2,20,0));
	
		//leftPanel contains audioPanel and StreamingPanel	
		Panel leftPanel = new Panel();
		leftPanel.setLayout(new BorderLayout());
		add(leftPanel);
		
		audioPanel = new AudioPanel(leftPanel);
		StreamingPanel streamingPanel = new StreamingPanel(leftPanel);
				
		//rightPanel contains videoPanel and buttonPanel		
		Panel rightPanel = new Panel();
		rightPanel.setLayout(new BorderLayout());
		add(rightPanel);
		
		videoPanel = new VideoPanel(rightPanel);
		final ButtonPanel buttonPanel = new ButtonPanel(rightPanel);
		
		setBackground(audioPanel.getBackground());
		
		

		
		//GUI Elements representing audio data
		audioCachingSpinner			= audioPanel.getAudioCachingSpinner();
		audioCachingSpinner.addChangeListener(this);
		
		audioBitRateSpinner			= audioPanel.getAudioBitRateSpinner();	
		audioBitRateSpinner.addChangeListener(this);
		
		audioCodecBox				= audioPanel.getAudioCodecBox();
		audioCodecBox.addActionListener(this);
		
		audioDeviceTextField		= audioPanel.getAudioDeviceTextField();
		audioDeviceTextField.addActionListener(this);
		
		audioProtocolTextField		= audioPanel.getAudioProtocolTextField();
		audioProtocolTextField.addActionListener(this);
		
		audioEnabledCheckBox		= audioPanel.getAudioEnabledCheckBox();
		audioEnabledCheckBox.addActionListener(this);
		
		audioSampleRateBox			= audioPanel.getAudioSampleRateBox();
		audioSampleRateBox.addActionListener(this);
		
		audioChannelSizeBox			= audioPanel.getAudioChannelSizeBox();
		audioChannelSizeBox.addActionListener(this);
		
		//GUI Elements representing video data
		videoCachingSpinner			= videoPanel.getVideoCachingSpinner();
		videoCachingSpinner.addChangeListener(this);
		
		videoBitRateSpinner			= videoPanel.getVideoBitRateSpinner();
		videoBitRateSpinner.addChangeListener(this);
		
		videoCodecBox				= videoPanel.getVideoCodecBox();
		videoCodecBox.addActionListener(this);
		
		videoDeviceTextField		= videoPanel.getVideoDeviceTextField();
		videoDeviceTextField.addActionListener(this);
		
		videoProtocolTextField		= videoPanel.getVideoProtocolTextField();
		videoProtocolTextField.addActionListener(this);
		
		videoEnabledCheckBox		= videoPanel.getVideoEnabledCheckBox();
		videoEnabledCheckBox.addActionListener(this);
		
		videoFrameRateSpinner		= videoPanel.getVideoFrameRateSpinner();
		videoFrameRateSpinner.addChangeListener(this);
		
		videoSizeWidthSpinner		= videoPanel.getVideoSizeWidthSpinner();
		videoSizeWidthSpinner.addChangeListener(this);
		
		videoSizeHeightSpinner		= videoPanel.getVideoSizeHeightSpinner();
		videoSizeHeightSpinner.addChangeListener(this);
		
		videoScaleBox				= videoPanel.getVideoScaleBox();
		videoScaleBox.addActionListener(this);
		
		//GUI Elements representing streaming data
		streamCachingSpinner		= streamingPanel.getStreamCachingSpinner();
		streamCachingSpinner.addChangeListener(this);
		
		streamIPTextField			= streamingPanel.getStreamIPTextField();
		streamIPTextField.addActionListener(this);
		
		streamProtocolBox			= streamingPanel.getStreamProtocol();
		streamProtocolBox.addActionListener(this);
		
		streamLocalPortSpinner		= streamingPanel.getStreamLocalPortSpinner();
		streamLocalPortSpinner.addChangeListener(this);
		
		streamTargetPortSpinner 	= streamingPanel.getStreamTargetPortSpinner();
		streamTargetPortSpinner.addChangeListener(this);
		
		//BUTTONs
		loadButton		= buttonPanel.getLoadButton();
		saveButton		= buttonPanel.getSaveButton();
		saveAsButton	= buttonPanel.getSaveAsButton();
		resetButton		= buttonPanel.getResetButton();
		
		//add Listeners to Buttons
		
		loadButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				
				Component c = ConfigurationPanel.this;
				while (!(c instanceof Frame)) c = c.getParent();
				
				if (fileChooser.showOpenDialog(c) == JFileChooser.APPROVE_OPTION) {//only if ok button was pressed
			        File selectedFile = fileChooser.getSelectedFile();
			        IConfiguration newconfig = StorageManager.getInstance().loadConfiguration(selectedFile.getAbsolutePath());
			        
			        if(newconfig == null || new VlcVerificator().verifyConfig(newconfig)!=null){
			        	//open an error dialog to the user
			        	JOptionPane.showMessageDialog(c,
			        			"ERROR during load configuration!",
							    "ERROR during load configuration!",
							    JOptionPane.ERROR_MESSAGE);
					}else{
						config.transfereValues(newconfig);
						//displaying the data from the config into the GUI Elements
						showConfigurationsData();
			        }
				}
			}
		});
		
		
		saveButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				if(!updateConfiguration()) return; //no action if there were no changes 
				//save config object into file
				StorageManager.getInstance().saveConfiguration(config);
			}
		});
		
		saveAsButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				if(!updateConfiguration()) return; //no action if there were no changes
				
				//select destination file where to store the config object
				Component c = ConfigurationPanel.this;
				while (!(c instanceof Frame)) c = c.getParent();
				//JFileChooser fileChooser = new JFileChooser();
				if (fileChooser.showSaveDialog(c) == JFileChooser.APPROVE_OPTION) {//only if ok button was pressed
			        String absolutePath =  fileChooser.getSelectedFile().getAbsolutePath();
			        		        
			        if(!absolutePath.endsWith(".properties")){absolutePath += ".properties";} 
			        	
			        //save config object into file
			        StorageManager.getInstance().saveConfiguration(config,absolutePath);
			        }
			}
		});
		
		resetButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				IConfigurator configurator = new V4l2Configurator();
				configurator.autoConfig(config);
				showConfigurationsData();
			}
		});
		
		//add listeners to checkboxes
		audioEnabledCheckBox.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				audioPanel.setEnabled(audioEnabledCheckBox.isSelected());
			}});
		
		videoEnabledCheckBox.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				videoPanel.setEnabled(videoEnabledCheckBox.isSelected());
			}});
	}

	/**
	 * displays the data from the configurationObject into the screen
	 */
	private void showConfigurationsData(){
		//GUI Elements representing audio data
		initFlag = true;
		audioCachingSpinner.setValue(config.getAudioCaching());
		audioBitRateSpinner.setValue(config.getAudioBitrate());
		audioCodecBox.setSelectedItem(config.getAudioCodec());
		audioDeviceTextField.setText(config.getAudioDevice());
		audioProtocolTextField.setText(config.getAudioProtocol());
		audioEnabledCheckBox.setSelected(config.isAudioEnabled());
		audioPanel.setEnabled(config.isAudioEnabled());
		audioSampleRateBox.setSelectedItem(config.getAudioSampleRate());
		audioChannelSizeBox.setSelectedItem(config.getAudioChannelSize());
		
		//GUI Elements representing video data
		videoCachingSpinner.setValue(config.getVideoCaching());
		videoBitRateSpinner.setValue(config.getVideoBitrate());
		videoCodecBox.setSelectedItem(config.getVideoCodec());
		videoDeviceTextField.setText(config.getVideoDevice());
		videoProtocolTextField.setText(config.getVideoProtocol());
		videoEnabledCheckBox.setSelected(config.isVideoEnabled());
		videoPanel.setEnabled(config.isVideoEnabled());
		videoFrameRateSpinner.setValue(config.getVideoFrameRate());
		videoSizeWidthSpinner.setValue(config.getVideoSize().width);
		videoSizeHeightSpinner.setValue(config.getVideoSize().height);
		videoScaleBox.setSelectedItem(config.getVideoScale());
		
		//GUI Elements representing streaming data
		streamCachingSpinner.setValue(config.getStreamCaching());
		streamIPTextField.setText(config.getStreamIp());
		streamProtocolBox.setSelectedItem(config.getStreamProtocol());
		streamLocalPortSpinner.setValue(config.getStreamLocalPort());
		streamTargetPortSpinner.setValue(config.getStreamTargetPort());
		initFlag=false;
	}
	
	
	
	/**
	 * 
	 * stores only the data that has been changed from the GUI Elements into the configuration object 
	 * @return true if any data has been changed otherwise false
	 */
	private boolean updateConfiguration(){
		//reset the errorMessageSum into the empty String
		errorMessageSum = "";
		
		//1. reading the values from the GUI Element and storing them into local variables
		int audioCaching = (Integer)audioCachingSpinner.getValue();
		int audioBitrate = (Integer)audioBitRateSpinner.getValue();
		String audioCodec = (String)audioCodecBox.getSelectedItem();
		String audioDevice = audioDeviceTextField.getText();
		String audioProtocol = audioProtocolTextField.getText();
		boolean audioEnabled = audioEnabledCheckBox.isSelected();
		int audioSampleRate = (Integer)audioSampleRateBox.getSelectedItem();
		int audioChannelSize = (Integer)audioChannelSizeBox.getSelectedItem();
		int videoCaching = 	(Integer)videoCachingSpinner.getValue();
		int videoBitrate =  (Integer)videoBitRateSpinner.getValue();
		String videoCodec = (String)videoCodecBox.getSelectedItem();
		String videoDevice = videoDeviceTextField.getText();
		String videoProtocol = videoProtocolTextField.getText();
		boolean videoEnabled = videoEnabledCheckBox.isSelected();
		int videoFrameRate = (Integer)videoFrameRateSpinner.getValue();
		Dimension videoSize = new Dimension((Integer)videoSizeWidthSpinner.getValue(),(Integer)videoSizeHeightSpinner.getValue());
		float videoScale = (Float)videoScaleBox.getSelectedItem();
		int streamCaching = (Integer)streamCachingSpinner.getValue();
		String ip = streamIPTextField.getText();
		String streamProtocol = (String)streamProtocolBox.getSelectedItem();
		int localPort = (Integer)streamLocalPortSpinner.getValue();
		int targetPort = (Integer)streamTargetPortSpinner.getValue();
			
		//2. verify the values with the IVerificator
		IVerificator verificator =  new VlcVerificator();
		//concatenate all the error messages into the String errorMessageSum 
		createErrorMessageSum(verificator.verifyAudioCaching(audioCaching));  
		createErrorMessageSum(verificator.verifyAudioBitrate(audioBitrate));
		createErrorMessageSum(verificator.verifyAudioChannelSize(audioChannelSize));
		createErrorMessageSum(verificator.verifyAudioCodec(audioCodec));
		createErrorMessageSum(verificator.verifyAudioDevice(audioDevice));
		createErrorMessageSum(verificator.verifyAudioProtocol(audioProtocol));
		createErrorMessageSum(verificator.verifyAudioSampleRate(audioSampleRate));
		createErrorMessageSum(verificator.verifyVideoBitrate(videoBitrate));
		createErrorMessageSum(verificator.verifyVideoCaching(videoCaching));
		createErrorMessageSum(verificator.verifyVideoCodec(videoCodec));
		createErrorMessageSum(verificator.verifyVideoDevice(videoDevice));
		createErrorMessageSum(verificator.verifyVideoFrameRate(videoFrameRate)); 
		createErrorMessageSum(verificator.verifyVideoProtocol(videoProtocol));
		createErrorMessageSum(verificator.verifyVideoScale(videoScale));
		createErrorMessageSum(verificator.verifyVideoSize(videoSize));
		createErrorMessageSum(verificator.verifyStreamCaching(streamCaching));
		createErrorMessageSum(verificator.verifyStreamIp(ip));
		createErrorMessageSum(verificator.verifyStreamLocalPort(localPort));
		createErrorMessageSum(verificator.verifyStreamProtocol(streamProtocol));
		createErrorMessageSum(verificator.verifyStreamTargetPort(targetPort));
		
		//if the verificator reports an errorMessage, do not continue but print the errorMessageSum to the user
		if(!errorMessageSum.equals("")){
			//get the reference to the current Frame
			Component c = ConfigurationPanel.this;
			while (!(c instanceof Frame)) c = c.getParent();
			JOptionPane.showMessageDialog(c,
				    errorMessageSum,
				    "Verification error",
				    JOptionPane.ERROR_MESSAGE);
			
			//load the values again from the configuration into the display
			showConfigurationsData();
			return false;
		} 
		
		//3. if the values are valid then check whether they and have been changed
			
		confirmMessage = "";
		boolean isAudioCachingChanged		= createConfirmMessage(config.getAudioCaching(),audioCaching,"audioCaching"); 
		boolean isAudioBitrateChanged		= createConfirmMessage(config.getAudioBitrate(),audioBitrate,"audioBitrate");
		boolean isAudioChannelSizeChanged	= createConfirmMessage(config.getAudioChannelSize(),audioChannelSize,"audioChannelSize");
		boolean isAudioCodecChanged			= createConfirmMessage(config.getAudioCodec(),audioCodec,"audioCodec");
		boolean isAudioDeviceChanged		= createConfirmMessage(config.getAudioDevice(),audioDevice,"audioDevice");
		boolean isAudioEnabledChanged		= createConfirmMessage(config.isAudioEnabled(),audioEnabled,"audioEnabled");
		boolean isAudioProtocolChanged		= createConfirmMessage(config.getAudioProtocol(),audioProtocol,"audioProtocol");
		boolean isAudioSampleRateChanged	= createConfirmMessage(config.getAudioSampleRate(),audioSampleRate,"audioSampleRate");
		boolean isVideoBitrateChanged		= createConfirmMessage(config.getVideoBitrate(),videoBitrate,"videoBitrate");
		boolean isVideoCachingChanged		= createConfirmMessage(config.getVideoCaching(),videoCaching,"videoCaching");
		boolean isVideoCodecChanged			= createConfirmMessage(config.getVideoCodec(),videoCodec,"videoCodec");
		boolean isVideoDeviceChanged		= createConfirmMessage(config.getVideoDevice(),videoDevice,"videoDevice");
		boolean isVideoEnabledChanged		= createConfirmMessage(config.isVideoEnabled(),videoEnabled,"videoEnabled");
		boolean isVideoFrameRateChanged		= createConfirmMessage(config.getVideoFrameRate(),videoFrameRate,"videoFrameRate");
		boolean isVideoProtocolChanged		= createConfirmMessage(config.getVideoProtocol(),videoProtocol,"videoProtocol");
		boolean isVideoScaleChanged			= createConfirmMessage(config.getVideoScale(),videoScale,"videoScale");
		boolean isVideoSizeChanged			= createConfirmMessage(config.getVideoSize(),videoSize,"videoSize");
		boolean isStreamCachingChanged		= createConfirmMessage(config.getStreamCaching(),streamCaching,"streamCaching");
		boolean isIPChangedChanged       	= createConfirmMessage(config.getStreamIp(),ip,"Stream IP");
		boolean isLocalPortChanged			= createConfirmMessage(config.getStreamLocalPort(),localPort,"localPort");
		boolean isStreamProtocolChanged		= createConfirmMessage(config.getStreamProtocol(),streamProtocol,"streamProtocol");
		boolean isTargetPortChanged			= createConfirmMessage(config.getStreamTargetPort(),targetPort,"targetPort");
		
		
		Component c = ConfigurationPanel.this;
		while (!(c instanceof Frame)) c = c.getParent();
		if(confirmMessage.equals("")){
			JOptionPane.showMessageDialog(c,"No data has been changed.\nNo storage required.");
			return false;
		}else{
			//int button = JOptionPane.showConfirmDialog((Frame)c,confirmMessage,"Please confirm the following changes!", JOptionPane.OK_CANCEL_OPTION);
			if(true){//store the new values into the confirugation object only if ok button was clicked 
				if(isAudioCachingChanged) 		config.setAudioCaching(audioCaching); 
				if(isAudioBitrateChanged) 		config.setAudioBitrate(audioBitrate);
				if(isAudioChannelSizeChanged)	config.setAudioChannelSize(audioChannelSize);
				if(isAudioCodecChanged)			config.setAudioCodec(audioCodec);
				if(isAudioDeviceChanged)		config.setAudioDevice(audioDevice);
				if(isAudioEnabledChanged)		config.setAudioEnabled(audioEnabled);
				if(isAudioProtocolChanged)		config.setAudioProtocol(audioProtocol);
				if(isAudioSampleRateChanged)	config.setAudioSampleRate(audioSampleRate);
				if(isVideoBitrateChanged)		config.setVideoBitrate(videoBitrate);
				if(isVideoCachingChanged)		config.setVideoCaching(videoCaching);
				if(isVideoCodecChanged)			config.setVideoCodec(videoCodec);
				if(isVideoDeviceChanged)		config.setVideoDevice(videoDevice);
				if(isVideoEnabledChanged)		config.setVideoEnabled(videoEnabled);
				if(isVideoFrameRateChanged) 	config.setVideoFrameRate(videoFrameRate);
				if(isVideoProtocolChanged)		config.setVideoProtocol(videoProtocol);
				if(isVideoScaleChanged)			config.setVideoScale(videoScale);
				if(isVideoSizeChanged)			config.setVideoSize(videoSize);
				if(isStreamCachingChanged)		config.setStreamCaching(streamCaching);
				if(isIPChangedChanged)			config.setStreamIp(ip);
				if(isLocalPortChanged)			config.setStreamLocalPort(localPort);
				if(isStreamProtocolChanged)		config.setStreamProtocol(streamProtocol);
				if(isTargetPortChanged)			config.setStreamTargetPort(targetPort);
			}
			
			return true;
		}
		
		
		
	}
	
	private void configUpdate()
	{
		int audioCaching = (Integer)audioCachingSpinner.getValue();
		int audioBitrate = (Integer)audioBitRateSpinner.getValue();
		String audioCodec = (String)audioCodecBox.getSelectedItem();
		String audioDevice = audioDeviceTextField.getText();
		String audioProtocol = audioProtocolTextField.getText();
		boolean audioEnabled = audioEnabledCheckBox.isSelected();
		int audioSampleRate = (Integer)audioSampleRateBox.getSelectedItem();
		int audioChannelSize = (Integer)audioChannelSizeBox.getSelectedItem();
		int videoCaching = 	(Integer)videoCachingSpinner.getValue();
		int videoBitrate =  (Integer)videoBitRateSpinner.getValue();
		String videoCodec = (String)videoCodecBox.getSelectedItem();
		String videoDevice = videoDeviceTextField.getText();
		String videoProtocol = videoProtocolTextField.getText();
		boolean videoEnabled = videoEnabledCheckBox.isSelected();
		int videoFrameRate = (Integer)videoFrameRateSpinner.getValue();
		Dimension videoSize = new Dimension((Integer)videoSizeWidthSpinner.getValue(),(Integer)videoSizeHeightSpinner.getValue());
		float videoScale = (Float)videoScaleBox.getSelectedItem();
		int streamCaching = (Integer)streamCachingSpinner.getValue();
		String ip = streamIPTextField.getText();
		String streamProtocol = (String)streamProtocolBox.getSelectedItem();
		int localPort = (Integer)streamLocalPortSpinner.getValue();
		int targetPort = (Integer)streamTargetPortSpinner.getValue();
		
		//2. verify the values with the IVerificator
		IVerificator verificator =  new VlcVerificator();
		//concatenate all the error messages into the String errorMessageSum 
		createErrorMessageSum(verificator.verifyAudioCaching(audioCaching));  
		createErrorMessageSum(verificator.verifyAudioBitrate(audioBitrate));
		createErrorMessageSum(verificator.verifyAudioChannelSize(audioChannelSize));
		createErrorMessageSum(verificator.verifyAudioCodec(audioCodec));
		createErrorMessageSum(verificator.verifyAudioDevice(audioDevice));
		createErrorMessageSum(verificator.verifyAudioProtocol(audioProtocol));
		createErrorMessageSum(verificator.verifyAudioSampleRate(audioSampleRate));
		createErrorMessageSum(verificator.verifyVideoBitrate(videoBitrate));
		createErrorMessageSum(verificator.verifyVideoCaching(videoCaching));
		createErrorMessageSum(verificator.verifyVideoCodec(videoCodec));
		createErrorMessageSum(verificator.verifyVideoDevice(videoDevice));
		createErrorMessageSum(verificator.verifyVideoFrameRate(videoFrameRate)); 
		createErrorMessageSum(verificator.verifyVideoProtocol(videoProtocol));
		createErrorMessageSum(verificator.verifyVideoScale(videoScale));
		createErrorMessageSum(verificator.verifyVideoSize(videoSize));
		createErrorMessageSum(verificator.verifyStreamCaching(streamCaching));
		createErrorMessageSum(verificator.verifyStreamIp(ip));
		createErrorMessageSum(verificator.verifyStreamLocalPort(localPort));
		createErrorMessageSum(verificator.verifyStreamProtocol(streamProtocol));
		createErrorMessageSum(verificator.verifyStreamTargetPort(targetPort));
		
		config.setAudioCaching(audioCaching); 
		config.setAudioBitrate(audioBitrate);
		config.setAudioChannelSize(audioChannelSize);
		config.setAudioCodec(audioCodec);
		config.setAudioDevice(audioDevice);
		config.setAudioEnabled(audioEnabled);
		config.setAudioProtocol(audioProtocol);
		config.setAudioSampleRate(audioSampleRate);
		config.setVideoBitrate(videoBitrate);
		config.setVideoCaching(videoCaching);
		config.setVideoCodec(videoCodec);
		config.setVideoDevice(videoDevice);
		config.setVideoEnabled(videoEnabled);
		config.setVideoFrameRate(videoFrameRate);
		config.setVideoProtocol(videoProtocol);
		config.setVideoScale(videoScale);
		config.setVideoSize(videoSize);
		config.setStreamCaching(streamCaching);
		config.setStreamIp(ip);
		config.setStreamLocalPort(localPort);
		config.setStreamProtocol(streamProtocol);
		config.setStreamTargetPort(targetPort);
	}
	
	private void createErrorMessageSum(String errorMessage){
		if (errorMessage != null) errorMessageSum += errorMessage +"\n";
	}
	
	private boolean createConfirmMessage(Object oldValue, Object newValue,String message){
		if(!oldValue.equals(newValue)){ 
			confirmMessage += message+" has been changed Old value: "+oldValue+" ---> New value: "+newValue+"\n";
			return true;
		}
		return false;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(!initFlag)
			configUpdate();
		
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		if(!initFlag)
			configUpdate();
		
	}

}
