package de.clab.osami.conferencing.vlc.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.Toolkit;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.processor.VlcCodecManager;

public class VideoPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final static Logger log = Logger.getLogger(VideoPanel.class);
	private JSpinner videoCachingSpinner;
	private JSpinner videoBitRateSpinner;
	private JComboBox videoCodecBox;
	private JTextField videoDeviceTextField;
	private JTextField videoProtocolTextField;
	private JCheckBox videoEnabledCheckBox;
	private JSpinner videoFrameRateSpinner;
	private JSpinner videoSizeWidthSpinner;
	private JSpinner videoSizeHeightSpinner;
	private JComboBox videoScaleBox;

	public VideoPanel(Panel parent){
		
		parent.add(this,BorderLayout.NORTH);
		setLayout(new GridLayout(10,2,0,5));
		setBorder(new TitledBorder("Video"));
		
		videoEnabledCheckBox = new JCheckBox();
		addGUIElement("VideoEnabled:"  ,videoEnabledCheckBox);
		
		videoCachingSpinner = new JSpinner(new SpinnerNumberModel(0,0,Integer.MAX_VALUE,1));
		addGUIElement("VideoCaching:"  ,videoCachingSpinner);
		
		videoBitRateSpinner = new JSpinner(new SpinnerNumberModel(0,0,32768,1));
		addGUIElement("VideoBitRate:"  ,videoBitRateSpinner);
		
		videoCodecBox = new JComboBox(VlcCodecManager.getInstance().getAvailableVideoCodecs());
		addGUIElement("VideoCodec:"    ,videoCodecBox);
		
		videoDeviceTextField = new JTextField();
		addGUIElement("VideoDevice:"   ,videoDeviceTextField);
		
		videoProtocolTextField = new JTextField();
		addGUIElement("VideoProtocol:" ,videoProtocolTextField);
		
		videoFrameRateSpinner = new JSpinner(new SpinnerNumberModel(0,0,99,1));
		addGUIElement("VideoFrameRate:",videoFrameRateSpinner);
		
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		videoSizeWidthSpinner = new JSpinner(new SpinnerNumberModel(0,0,screenSize.width,1));
		addGUIElement("Video Size Width:",videoSizeWidthSpinner);
		
		videoSizeHeightSpinner = new JSpinner(new SpinnerNumberModel(0,0,screenSize.height,1));
		addGUIElement("Video Size Height:",videoSizeHeightSpinner);
		
		videoScaleBox = new JComboBox(new Float[]{0.25f,0.5f,0.75f,1.0f,1.25f,1.5f,1.75f,2.0f});
		addGUIElement("VideoScale:"     ,videoScaleBox);
	}
	
	private void addGUIElement(String text, Component component){
		new GUIElement(this,text,component);
	}

	/**
	 * enable/disable if the audioEnabled checkBox has been selected/deselected
	 */
	@Override
	public void setEnabled(boolean isEnabled){
		/**enable all the child elements of this audiopanel except the first two (they are representing the checkbox itself*/
		Component[] components = getComponents();
		for(int i=2;i<components.length;i++)
			components[i].setEnabled(isEnabled);
	}
	
	//GETTERs
	public JSpinner getVideoCachingSpinner() {
		return videoCachingSpinner;
	}

	public JSpinner getVideoBitRateSpinner() {
		return videoBitRateSpinner;
	}

	public JComboBox getVideoCodecBox() {
		return videoCodecBox;
	}

	public JTextField getVideoDeviceTextField() {
		return videoDeviceTextField;
	}

	public JTextField getVideoProtocolTextField() {
		return videoProtocolTextField;
	}

	public JCheckBox getVideoEnabledCheckBox() {
		return videoEnabledCheckBox;
	}

	public JSpinner getVideoFrameRateSpinner() {
		return videoFrameRateSpinner;
	}

	public JSpinner getVideoSizeWidthSpinner() {
		return videoSizeWidthSpinner;
	}

	public JSpinner getVideoSizeHeightSpinner() {
		return videoSizeHeightSpinner;
	}

	public JComboBox getVideoScaleBox() {
		return videoScaleBox;
	}
}
