package de.clab.osami.conferencing.vlc.util;

import java.io.File;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import de.clab.osami.conferencing.vlc.config.IStorageManager;


public class Util {
	
	public static final String WINDOWS = "Windows";
	public static final String LINUX = "Linux";
	private final static Logger log = Logger.getLogger(Util.class);
		
	/**
	 * checks the operating system
	 * @return String
	 */
	public static String getOperatingSystem()
	{
		String osName = System.getProperty("os.name");
		log.info("Operating System is"+osName);
		
		return osName;
	}
	
	/**
	 * check if operating system is windows
	 * @return boolean
	 */
	public static boolean isWindows(){
		 
		String os = System.getProperty("os.name").toLowerCase();
		//windows
	    return (os.indexOf( "win" ) >= 0); 
 
	}
 
	/**
	 * checks if operating system is a mac
	 * @return boolean
	 */
	public static boolean isMac(){
 
		String os = System.getProperty("os.name").toLowerCase();
		//Mac
	    return (os.indexOf( "mac" ) >= 0); 
 
	}
 
	/**
	 * check if operating system is unix like
	 * @return boolean
	 */
	public static boolean isUnix(){
 
		String os = System.getProperty("os.name").toLowerCase();
		//linux or unix
	    return (os.indexOf( "nix") >=0 || os.indexOf( "nux") >=0);
 
	}

	
	/**
	 * Converts a String list into a String array
	 * @param list
	 * @return String[] 
	 */
	
	public static String[] listToArray(ArrayList<String> list)
	{
		String[] array = new String[list.size()];
		list.toArray(array);
		return array;
	}
	
	
	/**
	 * checks if a file given by a path exists
	 * e.g fileExists("./windowsConfiguration.properties");
	 * @param location path of the file
	 * @return true if file exists
	 */
	public static boolean fileExists(String location)
	{
	    File f = new File(location);
	      return f.exists();
	}
	
	public static String getLocalIp()
	{
		String ip = "127.0.0.1";
		try {
			ip = InetAddress.getLocalHost().getHostAddress();
			return ip;
		} catch (UnknownHostException e) {
			log.error("Unkown host. Couldn't find local ip");
		}
		return ip;
	}
	
	public static void cleanConfigurations()
	{
	
		
		File windowsConfigFile = new File(IStorageManager.WIN_PATH);
		if(windowsConfigFile.exists())
			windowsConfigFile.delete();
		
		File linuxConfigFile = new File(IStorageManager.LINUX_PATH);
		if(linuxConfigFile.exists())
			linuxConfigFile.delete();
	}
	
}
